/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * LaserInfo.cpp
 *
 *  Created on: Sep 17, 2010
 *      Author: ltwardon
 */

#define _USE_MATH_DEFINES
#include <math.h>

#include "LaserInfo.h"

using namespace airobots;

const map<LaserInfo::ScannerModel, LaserInfo> LaserInfo::info = LaserInfo::createInfo();

map<LaserInfo::ScannerModel, LaserInfo> LaserInfo::createInfo() {

	map<LaserInfo::ScannerModel,LaserInfo> info;
	info[LaserInfo::SICK_LMS200] = LaserInfo(
			180, // angle range
			361, // number of scans
			4, // max distance
			DEGREE,
			METER);

	info[LaserInfo::HOKUYO_UBG_04LX_F01] = LaserInfo(
			270, // angle range
			541, // number of scans
			4, // max distance
			DEGREE,
			METER);

	return info;

}

LaserInfo LaserInfo::getInfo(LaserInfo::ScannerModel model){
	return info.at(model);
}

int LaserInfo::calcScanIndex(LaserInfo::ScannerModel scannerModel, double angle) throw(OutOfRangeException){

	LaserInfo inf = info.at(scannerModel);
	// range of the laser scanner
	double rangeRad = inf.getRangeAngle(RADIAN);
	double rangeHalf = rangeRad / 2.0;

	// Convert target angle to scanner's scale
	double target = angle + rangeHalf;
	if (target < 0 || target > rangeRad) {
		throw OutOfRangeException("requested angle is out of laser range!");
	}

	// width of each scan
	double binWidth = rangeRad / (double) (inf.getNumScans() - 1);

	// calculate the requested scan and round it
	long targetScan = round(target / binWidth);

	return (int) targetScan;

}

LaserInfo::LaserInfo(){

}

LaserInfo::LaserInfo(double range, int numScans, double maxDistance,AngleUnit aU, LengthUnit lU) {
	this->range = UnitConverter::convert(range,aU,iAU);
	this->maxDistance = UnitConverter::convert(maxDistance,lU,iLU);
	this->numScans = numScans;
}

double LaserInfo::getRangeAngle(AngleUnit angleUnit) {
	return UnitConverter::convert(range,iAU, angleUnit);
}

double LaserInfo::getMaxDistance(LengthUnit lengthUnit) {
	return UnitConverter::convert(maxDistance,iLU,lengthUnit);
}

int LaserInfo::getNumScans() {
	return numScans;
}
