/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke,
 *              Lukas Kettenbach
 */

/* 
 * File:   Annotation.h
 * Author: lkettenb
 *
 * Created on April 18, 2012, 11:31 AM
 */

#ifndef ANNOTATION_H
#define	ANNOTATION_H

#include "../../Type.h"
#include "Viewpoint.h"
#include "../geometry/PrecisePolygon.h"
#include <list>

namespace airobots {

    /**
     * This type describes an annotation of a map, e.g., rooms or no-go areas. Each
     * annotation is represented by an area (polygon) and has several viewpoints of
     * different categories.
     *
     * <code>
     * <ANNOTATION label="kitchen">
     *      <TIMESTAMP>
     *          <INSERTED value="1334754163505" />
     *          <UPDATED value="1334754163505" />
     *      </TIMESTAMP>
     *      <GENERATOR>unknown</GENERATOR>
     *      <VIEWPOINT label="table1" category="VIEW">
     *          <ROBOTPOSITION> 
     *              <TIMESTAMP>
     *                  <INSERTED value="1334589211533" />
     *                  <UPDATED value="1334589211533" />
     *              </TIMESTAMP>
     *              <GENERATOR>unknown</GENERATOR>
     *              <POSITION x="0.0" y="1.0" theta="2.0" ref="world" kind="absolute" />
     *              </ROBOTPOSITION>
     *      </VIEWPOINT>
     *      <VIEWPOINT label="table2" category="VIEW">
     *          <ROBOTPOSITION>
     *              <TIMESTAMP>
     *                  <INSERTED value="1334589211533" />
     *                  <UPDATED value="1334589211533" />
     *              </TIMESTAMP>
     *              <GENERATOR>unknown</GENERATOR>
     *              <POSITION x="0.0" y="1.0" theta="2.0" ref="world" kind="absolute" />
     *          </ROBOTPOSITION>
     *      </VIEWPOINT>
     *      <PRECISEPOLYGON>
     *          <POINT2D x="0.0" y="0.0" scope="GLOBAL" />
     *          <POINT2D x="0.0" y="2000.0" scope="GLOBAL" />
     *          <POINT2D x="2000.0" y="2000.0" scope="GLOBAL" />
     *          <POINT2D x="2000.0" y="0.0" scope="GLOBAL" />
     *      </PRECISEPOLYGON>
     * </ANNOTATION>
     * </code>
     * 
     * @author lkettenb
     */
    class Annotation : public Type {
    public:

        /**
         * Default constructor (must be provided).
         * 
         * @see airobots::Type
         */
        Annotation() {
            label = "";
        }

        /**
         * Constructor.
         *
         * @param label Name of this annotation.
         * @param polygon Polygon that describes the area of this annotation.
         * @param viewpoints One or more initial viewpoints that belong to this
         * annotation.
         */
        Annotation(std::string label, airobots::PrecisePolygon polygon,
                std::list<airobots::Viewpoint> viewpoints);

        /**
         * Use this method to access the polygon of this annotation. You may use it
         * to determine whether a point is inside this polygon.
         *
         * @see airobots::PrecisePolygon#contains(double, double)
         *
         * @return Polygon of this annotation.
         */
        airobots::PrecisePolygon* getPolygon() {
            return &polygon;
        }

        /**
         * Returns a list of all viewpoints. Please note that a viewpoint is not
         * necessarily inside of the annotation its polygon/area.
         *
         * @return List of all viewpoints.
         */
        std::list<airobots::Viewpoint>* getViewpoints() {
            return &viewpoints;
        }

        /**
         * Label/name of this annotation.
         *
         * @return Label/name of this annotation.
         */
        std::string getLabel() {
            return label;
        }

        /*
         * Some setter.
         */

        void setLabel(std::string _label) {
            label = _label;
        }

        void setPolygon(airobots::PrecisePolygon _polygon) {
            polygon = _polygon;
        }

        void setViewpoints(std::list<airobots::Viewpoint> _viewpoints) {
            viewpoints = _viewpoints;
        }
        
        /**
         * Add a new viewpoint to the list of viewpoints.
         * 
         * @param _viewpoint Viewpoint to be added.
         */
        void addViewpoint(airobots::Viewpoint _viewpoint) {
            viewpoints.push_back(_viewpoint);
        }
        
        /**
         * Remove all viewpoints from list of viewpoints with the given label.
         * 
         * @param _label Label/name of the viewpoint to be removed.
         */
        void removeViewpoint(std::string _label) {
            for (std::list<airobots::Viewpoint>::iterator it = viewpoints.begin();
                    it != viewpoints.end();) {
                if ((*it).getLabel() == _label) {
                    it = viewpoints.erase(it);
                } else {
                     ++it;
                }
            }
        }
        
        /**
	 * Returns the Java class name of this entity.
	 *
	 * @return string
	 */
	static std::string getJavaClassName();

        /**
         * Getter for the XML base tag used for (de-)serialization.
         *
         * @return XML base tag.
         */
        static std::string getBaseTag();

        /**
         * Template string that can be used to construct an {@link xmltio::Location}.
         * @return Template string to construct an {@link xmltio::Location}.
         */
        static std::string getTemplate();



        /**
         * Converts this type to XML using the given location. The location must be
         * interpreted as already being parsed with the base tag for the type.
         *
         * @param location Location to serialize in.
         */
        void fillInto(xmltio::Location &location) const;

        /**
         * Converts a location containing an XML representation to the given type.
         *
         * @param location Location containing the XML representation to extract.
         * @param viewpoint Type instance to fill in the information found in the XML.
         */
        static void fromLocation(const xmltio::Location &location,
                Annotation &annotation);

        /**
         * Sanitizes the given location so that it can be used to construct a new
         * XML representation of this type. Therefore all information in the
         * template that is not managed by this type is preserved but every
         * information that could conflict with a new serialization of the type into
         * this template is removed.
         *
         * @param location The template its location to sanitize.
         * 
         * @see airobots::XmltioType
         */
        void sanitizeLocation(xmltio::Location &location) const;
    private:
        /**
         * Name/label of this annotation. This label should be unique. Please note
         * that this is not verified until now.
         */
        std::string label;
        /**
         * This polygon describes the area of this annotation.
         */
        airobots::PrecisePolygon polygon;
        /**
         * List of all viewpoints that belong to this annotation. Every
         * viewpoint should have a unique label. Please note that this is not 
         * verified until now.
         */
        std::list<airobots::Viewpoint> viewpoints;
    };

}

#endif	/* ANNOTATION_H */

