/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke,
 *               Lukas Kettenbach
 */

/* 
 * File:   Viewpoint.h
 * Author: lkettenb
 *
 * Created on April 17, 2012, 12:34 PM
 */

#ifndef VIEWPOINT_H
#define	VIEWPOINT_H

#include "../../Type.h"
#include "../navigation/PositionData.h"

namespace airobots {

    /**
     * Representation of a viewpoint. Each viewpoint has coordinates (x, y, yaw), a
     * prescribed category and a unique label.
     * 
     * <code>
     * <VIEWPOINT label="Bottle" category="OBJECT">
     *      <TIMESTAMP>
     *          <INSERTED value="1334740313587"/>
     *          <UPDATED value="1334740313587"/>
     *      </TIMESTAMP>
     * 
     *      <GENERATOR>unknown</GENERATOR>
     * 
     *      <ROBOTPOSITION>
     *          <TIMESTAMP>
     *              <INSERTED value="1334589075983" />
     *              <UPDATED value="1334589075983" />
     *          </TIMESTAMP>
     *          <GENERATOR>unknown</GENERATOR>
     *          <POSITION x="0.0" y="1.0" theta="2.0" ref="world" kind="absolute" />
     *      </ROBOTPOSITION>
     * </VIEWPOINT>
     * </code>
     *
     * @author lkettenb
     */
    class Viewpoint : public airobots::Type {
    public:

        /**
         * Three categories that a viewpoint can belong to.
         */
        enum ViewpointCategory {
            VIEW, OBJECT, PERSON 
        };

        /**
         * Default constructor (must be provided).
         * 
         * @see airobots::Type
         */
        Viewpoint() {
            category = VIEW;
            label = "";
            coordinates = airobots::PositionData(0.0f, 0.0f, 0.0f,
                    airobots::METER,
                    airobots::RADIAN);
        }

        /**
         * Constructor.
         *
         * @param coordinates Coordinates of the viewpoint (x, y, yaw).
         * @param category Category of this viewpoint.
         * @param label The unique label of this viewpoint.
         *
         * @see ViewpointCategory
         */
        Viewpoint(airobots::PositionData coordinates,
                ViewpointCategory category = VIEW,
                std::string label = "");

        /*
         * Some getter and setter.
         */

        ViewpointCategory getCategory() {
            return category;
        }

        void setCategory(ViewpointCategory _category) {
            category = _category;
        }

        airobots::PositionData getCoordinates() {
            return coordinates;
        }

        void setCoordinates(airobots::PositionData _coordinates) {
            coordinates = _coordinates;
        }

        std::string getLabel() {
            return label;
        }

        void setLabel(std::string _label) {
            label = _label;
        }


        /**
         * Getter for the XML base tag used for (de-)serialization.
         *
         * @return XML base tag.
         */
        static std::string getBaseTag();

        /**
         * Template string that can be used to construct an {@link xmltio::Location}.
         * @return Template string to construct an {@link xmltio::Location}.
         */
        static std::string getTemplate();

        /**
         * Converts this type to XML using the given location. The location must be
         * interpreted as already being parsed with the base tag for the type.
         *
         * @param location Location to serialize in.
         */
        void fillInto(xmltio::Location &location) const;

        /**
         * Converts a location containing an XML representation to the given type.
         *
         * @param location Location containing the XML representation to extract.
         * @param viewpoint Type instance to fill in the information found in the XML.
         */
        static void fromLocation(const xmltio::Location &location,
                Viewpoint &viewpoint);

        /**
         * Sanitizes the given location so that it can be used to construct a new
         * XML representation of this type. Therefore all information in the
         * template that is not managed by this type is preserved but every
         * information that could conflict with a new serialization of the type into
         * this template is removed.
         *
         * @param location The template its location to sanitize.
         * 
         * @see airobots::XmltioType
         */
        void sanitizeLocation(xmltio::Location &location) const;


    private:
        /**
         * Coordinates of this viewpoint (x, y, yaw).
         */
        airobots::PositionData coordinates;

        /**
         * The (optional) category of this viewpoint. 
         * Default: {@link ViewpointCategory#VIEW}.
         *
         * @see ViewpointCategory
         */
        ViewpointCategory category;

        /**
         * The label of this viewpoint. This label should be unique. Please note
         * that this is not verified until now.
         * 
         * ToDo: Assign a random string or number (in constructor).
         */
        std::string label;

    };

}

#endif	/* VIEWPOINT_H */

