/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * ROSCostmapConfiguration.h
 *
 *  Created on: May 31, 2012
 *      Author: pdressel
 */

#ifndef ROSCOSTMAPCONFIGURATION_H_
#define ROSCOSTMAPCONFIGURATION_H_

#include <stdio.h>
#include <string>
#include <xmltio/xmltio.hpp>
#include "ROSDynamicConfiguration.h"

namespace airobots 
{

class ROSCostmapConfiguration : public ROSDynamicConfiguration 
{
    
public:
    
    /**
     * ROSCostmapConfiguration constructor. Make sure the member variables are 
     * set with the current configuration as is on the corresponding ROS server.
     * @param isGlobalCostmap 
     * Setting this parameter to true allows for manipulation of global_costmap
     * parameters. Setting this parameter to false allows for manipulation of 
     * local_costmap parameters. The default value is 'true'.
     */
    ROSCostmapConfiguration(const bool isGlobalCostmap = true);
    virtual ~ROSCostmapConfiguration() {};
    
    /**
     * Explanation: Specifies the delay in transform (tf) data that is tolerable
     * in seconds. This parameter serves as a safeguard to losing a link in the
     * tf tree while still allowing an amount of latency the user is comfortable
     * with to exist in the system. For example, a transform being 0.2 seconds
     * out-of-date may be tolerable, but a transform being 8 seconds out of date
     * is not. If the tf transform between the coordinate frames specified by
     * the global_frame and robot_base_frame parameters is transform_tolerance
     * seconds older than ros::Time::now(), then the navigation stack will stop
     * the robot.
     * ROS topic name: '<node_name>/transform_tolerance
     * Type and default: double, 0.3 (0.2 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 10.0
     * @return the transform_tolerance
     */    
    double getTransformTolerance() const {
        return this->transformTolerance;
    }

    /**
     * Explanation: Specifies the delay in transform (tf) data that is tolerable
     * in seconds. This parameter serves as a safeguard to losing a link in the
     * tf tree while still allowing an amount of latency the user is comfortable
     * with to exist in the system. For example, a transform being 0.2 seconds
     * out-of-date may be tolerable, but a transform being 8 seconds out of date
     * is not. If the tf transform between the coordinate frames specified by
     * the global_frame and robot_base_frame parameters is transform_tolerance
     * seconds older than ros::Time::now(), then the navigation stack will stop
     * the robot.
     * ROS topic name: '<node_name>/transform_tolerance
     * Type and default: double, 0.3 (0.2 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 10.0
     * @param transform_tolerance the transform_tolerance to set
     */
    void setTransformTolerance(double transform_tolerance) {
        this->transformTolerance = transform_tolerance;
    }

    /**
     * Explanation: The frequency in Hz for the map to be updated.
     * ROS topic name: '<node_name>/update_frequency
     * Type and default: double, 5.0
     * Min, Max value: 0.0, 100.0
     * @return the update_frequency
     */
    double getUpdateFrequency() const {
        return this->updateFrequency;
    }

    /**
     * Explanation: The frequency in Hz for the map to be updated.
     * ROS topic name: '<node_name>/update_frequency
     * Type and default: double, 5.0
     * Min, Max value: 0.0, 100.0
     * @param update_frequency the update_frequency to set
     */
    void setUpdateFrequency(double update_frequency) {
        this->updateFrequency = update_frequency;
    }

    /**
     * Explanation: The frequency in Hz for the map to publish display
     * information.
     * ROS topic name: '<node_name>/publish_frequency
     * Type and default: double, 2.0 (0.0 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 100.0
     * @return the publish_frequency
     */
    double getPublishFrequency() const {
        return this->publishFrequency;
    }

    /**
     * Explanation: The frequency in Hz for the map to publish display
     * information.
     * ROS topic name: '<node_name>/publish_frequency
     * Type and default: double, 2.0 (0.0 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 100.0
     * @param publish_frequency the publish_frequency to set
     */
    void setPublishFrequency(double publish_frequency) {
        this->publishFrequency = publish_frequency;
    }

    /**
     * Explanation: The maximum height in meters of a sensor reading considered
     * valid. This is usually set to be slightly higher than the height of the
     * robot.
     * ROS topic name: '<node_name>/max_obstacle_height
     * Type and default: double, 2.0
     * Min, Max value: 0.0, 50.0
     * @return the max_obstacle_height
     */
    double getMaxObstacleHeight() const {
        return this->maxObstacleHeight;
    }

    /**
     * Explanation: The maximum height in meters of a sensor reading considered
     * valid. This is usually set to be slightly higher than the height of the
     * robot.
     * ROS topic name: '<node_name>/max_obstacle_height
     * Type and default: double, 2.0
     * Min, Max value: 0.0, 50.0
     * @param max_obstacle_height the max_obstacle_height to set
     */
    void setMaxObstacleHeight(double max_obstacle_height) {
        this->maxObstacleHeight = max_obstacle_height;
    }

    /**
     * Explanation: The maximum range in meters at which to insert obstacles
     * into the costmap using sensor data.
     * ROS topic name: '<node_name>/max_obstacle_range
     * Type and default: double, 2.5
     * Min, Max value: 0.0, 50.0
     * @return the max_obstacle_range
     */
    double getMaxObstacleRange() const {
        return this->maxObstacleRange;
    }

    /**
     * Explanation: The maximum range in meters at which to insert obstacles
     * into the costmap using sensor data.
     * ROS topic name: '<node_name>/max_obstacle_range
     * Type and default: double, 2.5
     * Min, Max value: 0.0, 50.0
     * @param max_obstacle_range the max_obstacle_range to set
     */
    void setMaxObstacleRange(double max_obstacle_range) {
        this->maxObstacleRange = max_obstacle_range;
    }

    /**
     * Explanation: The maximum range in meters at which to raytrace out
     * obstacles from the map using sensor data. This can be overridden on a
     * per-sensor basis.
     * ROS topic name: '<node_name>/raytrace_range
     * Type and default: double, 3.0
     * Min, Max value: 0.0, 50.0
     * @return the raytrace_range
     */
    double getRaytraceRange() const {
        return this->raytraceRange;
    }

    /**
     * Explanation: The maximum range in meters at which to raytrace out
     * obstacles from the map using sensor data. This can be overridden on a
     * per-sensor basis.
     * ROS topic name: '<node_name>/raytrace_range
     * Type and default: double, 3.0
     * Min, Max value: 0.0, 50.0
     * @param raytrace_range the raytrace_range to set
     */
    void setRaytraceRange(double raytrace_range) {
        this->raytraceRange = raytrace_range;
    }

    /**
     * Explanation: A scaling factor to apply to cost values during inflation.
     * The cost function is computed as follows for all cells in the costmap
     * further than the inscribed radius distance and closer than the inflation
     * radius distance away from an actual obstacle:
     * exp(-1.0 * cost_scaling_factor * (distance_from_obstacle - inscribed_radius))
     * * (costmap_2d::INSCRIBED_INFLATED_OBSTACLE - 1)
     * where costmap_2d::INSCRIBED_INFLATED_OBSTACLE is currently 254.
     * ROS topic name: '<node_name>/cost_scaling_factor
     * Type and default: double, 10.0
     * Min, Max value: 0.0, 100.0
     * @return the cost_scaling_factor
     */
    double getCostScalingFactor() const {
        return this->costScalingFactor;
    }

    /**
     * Explanation: A scaling factor to apply to cost values during inflation.
     * The cost function is computed as follows for all cells in the costmap
     * further than the inscribed radius distance and closer than the inflation
     * radius distance away from an actual obstacle:
     * exp(-1.0 * cost_scaling_factor * (distance_from_obstacle - inscribed_radius))
     * * (costmap_2d::INSCRIBED_INFLATED_OBSTACLE - 1)
     * where costmap_2d::INSCRIBED_INFLATED_OBSTACLE is currently 254.
     * ROS topic name: '<node_name>/cost_scaling_factor
     * Type and default: double, 10.0
     * Min, Max value: 0.0, 100.0
     * @param cost_scaling_factor the cost_scaling_factor to set
     */
    void setCostScalingFactor(double cost_scaling_factor) {
        this->costScalingFactor = cost_scaling_factor;
    }

    /**
     * Explanation: The radius in meters to which the map inflates obstacle
     * cost values.
     * ROS topic name: '<node_name>/inflation_radius
     * Type and default: double, 0.45 (0.55 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 50.0
     * @return the inflation_radius
     */
    double getInflationRadius() const {
        return this->inflationRadius;
    }

    /**
     * Explanation: The radius in meters to which the map inflates obstacle
     * cost values.
     * ROS topic name: '<node_name>/inflation_radius
     * Type and default: double, 0.45 (0.55 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 50.0
     * @param inflation_radius the inflation_radius to set
     */
    void setInflationRadius(double inflation_radius) {
        this->inflationRadius = inflation_radius;
    }

    /**
     * Explanation: The footprint of the robot specified in the robot_base_frame
     * coordinate frame as a list in the format:
     * [[x1,y1],[x2,y2],...,[xn,yn]]. The footprint specification assumes the
     * center point of the robot is at (0.0, 0.0) in the robot_base_frame and
     * that the points are specified in meters, both clockwise and
     * counter-clockwise orderings of points are supported.
     * ROS topic name: '<node_name>/footprint
     * Type and default: string, '[]'
     * @return the footprint
     */
    std::string getFootprint() const {
        return this->footprint;
    }

    /**
     * Explanation: The footprint of the robot specified in the robot_base_frame
     * coordinate frame as a list in the format:
     * [[x1,y1],[x2,y2],...,[xn,yn]]. The footprint specification assumes the
     * center point of the robot is at (0.0, 0.0) in the robot_base_frame and
     * that the points are specified in meters, both clockwise and
     * counter-clockwise orderings of points are supported.
     * ROS topic name: '<node_name>/footprint
     * Type and default: string, '[]'
     * @param footprint the footprint to set
     */
    void setFootprint(std::string footprint) {
        this->footprint = footprint;
    }

    /**
     * This parameter is overwritten if the "footprint" parameter is set.
     * Explanation: The radius of the robot in meters , this parameter should
     * only be set for circular robots, all others should use the "footprint"
     * parameter described above.
     * ROS topic name: '<node_name>/robot_radius
     * Type and default: double, 0.30 (0.46 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 10.0
     * @return the robot_radius
     */
    double getRobotRadius() const {
        return this->robotRadius;
    }

    /**
     * This parameter is overwritten if the "footprint" parameter is set.
     * Explanation: The radius of the robot in meters , this parameter should
     * only be set for circular robots, all others should use the "footprint"
     * parameter described above.
     * ROS topic name: '<node_name>/robot_radius
     * Type and default: double, 0.30 (0.46 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 10.0
     * @param robot_radius the robot_radius to set
     */
    void setRobotRadius(double robot_radius) {
        this->robotRadius = robot_radius;
    }

    /**
     * @return the static_map
     */
    bool isStaticMap() const {
        return this->staticMap;
    }

    /**
     * @param static_map the static_map to set
     */
    void setStaticMap(bool static_map) {
        this->staticMap = static_map;
    }

    /**
     * Explanation: Whether or not to use a rolling window version of the
     * costmap. If the static_map parameter is set to true, this parameter
     * must be set to false.
     * ROS topic name: '<node_name>/rolling_window
     * Type and default: bool, true (false in section 3.2 in ROS wiki)
     * @return the rolling_window
     */
    bool isRollingWindow() const {
        return this->rollingWindow;
    }

    /**
     * Explanation: Whether or not to use a rolling window version of the
     * costmap. If the static_map parameter is set to true, this parameter
     * must be set to false.
     * ROS topic name: '<node_name>/rolling_window
     * Type and default: bool, true (false in section 3.2 in ROS wiki)
     * @param rolling_window the rolling_window to set
     */
    void setRollingWindow(bool rolling_window) {
        this->rollingWindow = rolling_window;
    }

    /**
     * Explanation: The value for which a cost should be considered unknown when
     * reading in a map from the map server. If the costmap is not tracking
     * unknown space, costs of this value will be considered occupied. A value
     * of zero also results in this parameter being unused.
     * ROS topic name: '<node_name>/unknown_cost_value
     * Type and default: int, 0
     * Min, Max value: 0, 255
     * @return the unknown_cost_value
     */
    int getUnknownCostValue() const {
        return this->unknownCostValue;
    }

    /**
     * Explanation: The value for which a cost should be considered unknown when
     * reading in a map from the map server. If the costmap is not tracking
     * unknown space, costs of this value will be considered occupied. A value
     * of zero also results in this parameter being unused.
     * ROS topic name: '<node_name>/unknown_cost_value
     * Type and default: int, 0
     * Min, Max value: 0, 255
     * @param unknown_cost_value the unknown_cost_value to set
     */
    void setUnknownCostValue(int unknown_cost_value) {
        this->unknownCostValue = unknown_cost_value;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The width of the map in meters.
     * ROS topic name: '<node_name>/width
     * Type and default: int, 6 (10 in section 3.2 in ROS wiki)
     * Min, Max value: 0, 20
     * @return the width
     */
    int getWidth() const {
        return this->width;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The width of the map in meters.
     * ROS topic name: '<node_name>/width
     * Type and default: int, 6 (10 in section 3.2 in ROS wiki)
     * Min, Max value: 0, 20
     * @param width the width to set
     */
    void setWidth(int width) {
        this->width = width;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The height of the map in meters.
     * ROS topic name: '<node_name>/height
     * Type and default: int, 6 (10 in section 3.2 in ROS wiki)
     * Min, Max value: 0, 20
     * @return the height
     */
    int getHeight() const {
        return this->height;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The height of the map in meters.
     * ROS topic name: '<node_name>/height
     * Type and default: int, 6 (10 in section 3.2 in ROS wiki)
     * Min, Max value: 0, 20
     * @param height the height to set
     */
    void setHeight(int height) {
        this->height = height;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The resolution of the map in meters/cell.
     * ROS topic name: '<node_name>/resolution
     * Type and default: double, 0.05
     * Min, Max value: 0.0, 50.0
     * @return the resolution
     */
    double getResolution() const {
        return this->resolution;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The resolution of the map in meters/cell.
     * ROS topic name: '<node_name>/resolution
     * Type and default: double, 0.05
     * Min, Max value: 0.0, 50.0
     * @param resolution the resolution to set
     */
    void setResolution(double resolution) {
        this->resolution = resolution;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The x origin of the map in the global frame in meters.
     * ROS topic name: '<node_name>/origin_x
     * Type and default: double, 0.0
     * Min, Max value: 0.0, inf
     * @return the origin_x
     */
    double getOriginX() const {
        return this->originX;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The x origin of the map in the global frame in meters.
     * ROS topic name: '<node_name>/origin_x
     * Type and default: double, 0.0
     * Min, Max value: 0.0, inf
     * @param origin_x the origin_x to set
     */
    void setOriginX(double origin_x) {
        this->originX = origin_x;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The y origin of the map in the global frame in meters.
     * ROS topic name: '<node_name>/origin_y
     * Type and default: double, 0.0
     * Min, Max value: -inf, inf
     * @return the origin_y
     */
    double getOriginY() const {
        return this->originY;
    }

    /**
     * This parameter is overwritten if "static_map" is set to true, and its
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The y origin of the map in the global frame in meters.
     * ROS topic name: '<node_name>/origin_y
     * Type and default: double, 0.0
     * Min, Max value: -inf, inf
     * @param origin_y the origin_y to set
     */
    void setOriginY(double origin_y) {
        this->originY = origin_y;
    }

    /**
     * Explanation: Whether or not to publish the underlying voxel grid for
     * visualization purposes.
     * ROS topic name: '<node_name>/publish_voxel_map
     * Type and default: bool, false
     * @return the publishVoxelMap
     */
    bool isPublishVoxelMap() const {
        return this->publishVoxelMap;
    }

    /**
     * Explanation: Whether or not to publish the underlying voxel grid for
     * visualization purposes.
     * ROS topic name: '<node_name>/publish_voxel_map
     * Type and default: bool, false
     * @param publishVoxelMap the publishVoxelMap to set
     */
    void setPublishVoxelMap(bool publishVoxelMap) {
        this->publishVoxelMap = publishVoxelMap;
    }

    /**
     * Explanation: The threshold value at which to consider a cost lethal when
     * reading in a map from the map server.
     * ROS topic name: '<node_name>/lethal_cost_threshold
     * Type and default: int, 100
     * Min, Max value: 0, 255
     * @return the lethalCostThreshold
     */
    int getLethalCostThreshold() const {
        return this->lethalCostThreshold;
    }

    /**
     * Explanation: The threshold value at which to consider a cost lethal when
     * reading in a map from the map server.
     * ROS topic name: '<node_name>/lethal_cost_threshold
     * Type and default: int, 100
     * Min, Max value: 0, 255
     * @param lethalCostThreshold the lethalCostThreshold to set
     */
    void setLethalCostThreshold(int lethalCostThreshold) {
        this->lethalCostThreshold = lethalCostThreshold;
    }

    /**
     * Explanation: The topic that the costmap subscribes to for the static map.
     * This parameter is useful when you have multiple costmap instances within
     * a single node that you want to use different static maps.
     * ROS topic name: '<node_name>/map_topic'.
     * Type and default: string, 'map'
     * @return the mapTopic
     */
    std::string getMapTopic() const {
        return this->mapTopic;
    }

    /**
     * Explanation: The topic that the costmap subscribes to for the static map.
     * This parameter is useful when you have multiple costmap instances within
     * a single node that you want to use different static maps.
     * ROS topic name: '<node_name>/map_topic'.
     * Type and default: string, 'map'
     * @param mapTopic the mapTopic to set
     */
    void setMapTopic(std::string mapTopic) {
        this->mapTopic = mapTopic;
    }

    /**
     * Explanation: What map type to use. "voxel" or "costmap" are the supported
     * types, with the difference between them being a 3D-view of the world vs.
     * a 2D-view of the world.
     * ROS topic name: '<node_name>/map_type'.
     * Type and default: string, 'costmap' ('voxel' in section 3.2 in ROS wiki)
     * @return the mapType
     */
    std::string getMapType() const {
        return this->mapType;
    }

    /**
     * Explanation: What map type to use. "voxel" or "costmap" are the supported
     * types, with the difference between them being a 3D-view of the world vs.
     * a 2D-view of the world.
     * ROS topic name: '<node_name>/map_type'.
     * Type and default: string, 'costmap' ('voxel' in section 3.2 in ROS wiki)
     * @param mapType the mapType to set
     */
    void setMapType(std::string mapType) {
        this->mapType = mapType;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The z origin of the map in meters.
     * ROS topic name: '<node_name>/origin_z'.
     * Type and default: double, 0.0
     * Min, Max value: 0.0, inf
     * @return the zOrigin
     */
    double getzOrigin() const {
        return this->zOrigin;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The z origin of the map in meters.
     * ROS topic name: '<node_name>/origin_z'.
     * Type and default: double, 0.0
     * Min, Max value: 0.0, inf
     * @param zOrigin the zOrigin to set
     */
    void setzOrigin(double zOrigin) {
        this->zOrigin = zOrigin;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The z resolution of the map in meters/cell.
     * ROS topic name: '<node_name>/z_resolution'.
     * Type and default: double, 0.2
     * Min, Max value: 0.0, 50.0
     * @return the zResolution
     */
    double getzResolution() const {
        return this->zResolution;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The z resolution of the map in meters/cell.
     * ROS topic name: '<node_name>/z_resolution'.
     * Type and default: double, 0.2
     * Min, Max value: 0.0, 50.0
     * @param zResolution the zResolution to set
     */
    void setzResolution(double zResolution) {
        this->zResolution = zResolution;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The number of voxels to in each vertical column, the height
     * of the grid is z_resolution * z_voxels.
     * ROS topic name: '<node_name>/z_voxels'.
     * Type and default: int, 10
     * Min, Max value: 0, 16
     * @return the zVoxels
     */
    int getzVoxels() const {
        return this->zVoxels;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The number of voxels to in each vertical column, the height
     * of the grid is z_resolution * z_voxels.
     * ROS topic name: '<node_name>/z_voxels'.
     * Type and default: int, 10
     * Min, Max value: 0, 16
     * @param zVoxels the zVoxels to set
     */
    void setzVoxels(int zVoxels) {
        this->zVoxels = zVoxels;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The number of unknown cells allowed in a column considered
     * to be "known".
     * ROS topic name: '<node_name>/unknown_threshold'.
     * Type and default: int, 15
     * Min, Max value: 0, 16
     * @return the unknownThreshold
     */
    int getUnknownThreshold() const {
        return this->unknownThreshold;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The number of unknown cells allowed in a column considered
     * to be "known".
     * ROS topic name: '<node_name>/unknown_threshold'.
     * Type and default: int, 15
     * Min, Max value: 0, 16
     * @param unknownThreshold the unknownThreshold to set
     */
    void setUnknownThreshold(int unknownThreshold) {
        this->unknownThreshold = unknownThreshold;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The maximum number of marked cells allowed in a column
     * considered to be "free".
     * ROS topic name: '<node_name>/mark_threshold'.
     * Type and default: int, 0
     * Min, Max value: 0, 16
     * @return the markThreshold
     */
    int getMarkThreshold() const {
        return this->markThreshold;
    }

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The maximum number of marked cells allowed in a column
     * considered to be "free".
     * ROS topic name: '<node_name>/mark_threshold'.
     * Type and default: int, 0
     * Min, Max value: 0, 16
     * @param markThreshold the markThreshold to set
     */
    void setMarkThreshold(int markThreshold) {
        this->markThreshold = markThreshold;
    }

    /**
     * This parameter is only used if map_type is set to "costmap".
     * Explanation: Specifies whether or not to track what space in the costmap
     * is unknown, meaning that no observation about a cell has been seen from
     * any sensor source.
     * ROS topic name: '<node_name>/track_unknown_space'.
     * Type and default: bool, false
     * @return the trackUnknownSpace
     */
    bool isTrackUnknownSpace() const {
        return this->trackUnknownSpace;
    }

    /**
     * This parameter is only used if map_type is set to "costmap".
     * Explanation: Specifies whether or not to track what space in the costmap
     * is unknown, meaning that no observation about a cell has been seen from
     * any sensor source.
     * ROS topic name: '<node_name>/track_unknown_space'.
     * Type and default: bool, false
     * @param trackUnknownSpace the trackUnknownSpace to set
     */
    void setTrackUnknownSpace(bool trackUnknownSpace) {
        this->trackUnknownSpace = trackUnknownSpace;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults.
     * ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @return the restoreDefaults
     */
    bool isRestoreDefaults() const {
        return this->restoreDefaults;
    }

    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration
     * server to its defaults.
     * ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     * @param restoreDefaults the restoreDefaults to set
     */
    void setRestoreDefaults(bool restoreDefaults) {
        this->restoreDefaults = restoreDefaults;
    }

    /**
     * Non ROS configuratin parameter!
     * Setting this parameter to true allows for manipulation of global_costmap
     * parameters. Setting this parameter to false allows for manipulation of
     * local_costmap parameters. The default value is 'true'.
     * @return the isGlobalCostmap
     */
    bool isIsGlobalCostmap() const {
        return this->isGlobalCostmap;
    }

    /**
     * Non ROS configuratin parameter!
     * Setting this parameter to true allows for manipulation of global_costmap
     * parameters. Setting this parameter to false allows for manipulation of
     * local_costmap parameters. The default value is 'true'.
     * @param isGlobalCostmap the isGlobalCostmap to set
     */
    void setIsGlobalCostmap(bool isGlobalCostmap) {
        this->isGlobalCostmap = isGlobalCostmap;
    }        

    static std::string getBaseTag();
    static std::string getTemplate();
    
    void sanitizeLocation(xmltio::Location &loc) const;
    void fillInto(xmltio::Location &loc) const;
    static void fromLocation(const xmltio::Location &loc, ROSCostmapConfiguration &config);
        
    static const std::string TRANSFORM_TOLERANCE_ATTRIBUTE;
    static const std::string UPDATE_FREQUENCY_ATTRIBUTE;
    static const std::string PUBLISH_FREQUENCY_ATTRIBUTE;
    static const std::string MAX_OBSTACLE_HEIGHT_ATTRIBUTE;
    static const std::string MAX_OBSTACLE_RANGE_ATTRIBUTE;
    static const std::string RAYTRACE_RANGE_ATTRIBUTE;
    static const std::string COST_SCALING_FACTOR_ATTRIBUTE;
    static const std::string INFLATION_RADIUS_ATTRIBUTE;
    static const std::string FOOTPRINT_ATTRIBUTE;
    static const std::string ROBOT_RADIUS_ATTRIBUTE;
    static const std::string STATIC_MAP_ATTRIBUTE;
    static const std::string ROLLING_WINDOW_ATTRIBUTE;
    static const std::string UNKNOWN_COST_VALUE_ATTRIBUTE;
    static const std::string WIDTH_ATTRIBUTE;
    static const std::string HEIGHT_ATTRIBUTE;
    static const std::string RESOLUTION_ATTRIBUTE;
    static const std::string ORIGIN_X_ATTRIBUTE;
    static const std::string ORIGIN_Y_ATTRIBUTE;
    static const std::string PUBLISH_VOXEL_MAP_ATTRIBUTE;
    static const std::string LETHAL_COST_THRESHOLD_ATTRIBUTE;
    static const std::string MAP_TOPIC_ATTRIBUTE;
    static const std::string MAP_TYPE_ATTRIBUTE;
    static const std::string Z_ORIGIN_ATTRIBUTE;
    static const std::string Z_RESOLUTION_ATTRIBUTE;
    static const std::string Z_VOXELS_ATTRIBUTE;
    static const std::string UNKNOWN_THRESHOLD_ATTRIBUTE;
    static const std::string MARK_THRESHOLD_ATTRIBUTE;
    static const std::string TRACK_UNKNOWN_SPACE_ATTRIBUTE;
    static const std::string RESTORE_DEFAULTS_ATTRIBUTE;
    
    static const std::string IS_GLOBAL_COSTMAP_ATTRIBUTE;
    
private:
    
    static const std::string COSTMAP_CONFIGURATION_TAG;
    
    /**
     * Explanation: Specifies the delay in transform (tf) data that is tolerable
     * in seconds. This parameter serves as a safeguard to losing a link in the 
     * tf tree while still allowing an amount of latency the user is comfortable 
     * with to exist in the system. For example, a transform being 0.2 seconds 
     * out-of-date may be tolerable, but a transform being 8 seconds out of date 
     * is not. If the tf transform between the coordinate frames specified by 
     * the global_frame and robot_base_frame parameters is transform_tolerance 
     * seconds older than ros::Time::now(), then the navigation stack will stop 
     * the robot. 
     * ROS topic name: '<node_name>/transform_tolerance
     * Type and default: double, 0.3 (0.2 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 10.0
     */                        
    double transformTolerance;
    
    /**
     * Explanation: The frequency in Hz for the map to be updated.
     * ROS topic name: '<node_name>/update_frequency
     * Type and default: double, 5.0
     * Min, Max value: 0.0, 100.0
     */                        
    double updateFrequency;
    
    /**
     * Explanation: The frequency in Hz for the map to publish display 
     * information. 
     * ROS topic name: '<node_name>/publish_frequency
     * Type and default: double, 2.0 (0.0 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 100.0
     */                    
    double publishFrequency;
    
    /**
     * Explanation: The maximum height in meters of a sensor reading considered 
     * valid. This is usually set to be slightly higher than the height of the 
     * robot.
     * ROS topic name: '<node_name>/max_obstacle_height
     * Type and default: double, 2.0
     * Min, Max value: 0.0, 50.0
     */                
    double maxObstacleHeight;
    
    /**
     * Explanation: The maximum range in meters at which to insert obstacles 
     * into the costmap using sensor data.
     * ROS topic name: '<node_name>/max_obstacle_range
     * Type and default: double, 2.5
     * Min, Max value: 0.0, 50.0
     */            
    double maxObstacleRange;
    
    /**
     * Explanation: The maximum range in meters at which to raytrace out 
     * obstacles from the map using sensor data. This can be overridden on a 
     * per-sensor basis.
     * ROS topic name: '<node_name>/raytrace_range
     * Type and default: double, 3.0
     * Min, Max value: 0.0, 50.0
     */            
    double raytraceRange;

    /**
     * Explanation: A scaling factor to apply to cost values during inflation. 
     * The cost function is computed as follows for all cells in the costmap 
     * further than the inscribed radius distance and closer than the inflation 
     * radius distance away from an actual obstacle:
     * exp(-1.0 * cost_scaling_factor * (distance_from_obstacle - inscribed_radius)) 
     * * (costmap_2d::INSCRIBED_INFLATED_OBSTACLE - 1) 
     * where costmap_2d::INSCRIBED_INFLATED_OBSTACLE is currently 254. 
     * ROS topic name: '<node_name>/cost_scaling_factor
     * Type and default: double, 10.0
     * Min, Max value: 0.0, 100.0
     */                
    double costScalingFactor;
    
    /**
     * Explanation: The radius in meters to which the map inflates obstacle 
     * cost values.
     * ROS topic name: '<node_name>/inflation_radius
     * Type and default: double, 0.45 (0.55 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 50.0
     */            
    double inflationRadius;
    
    /**
     * Explanation: The footprint of the robot specified in the robot_base_frame
     * coordinate frame as a list in the format: 
     * [[x1,y1],[x2,y2],...,[xn,yn]]. The footprint specification assumes the 
     * center point of the robot is at (0.0, 0.0) in the robot_base_frame and 
     * that the points are specified in meters, both clockwise and 
     * counter-clockwise orderings of points are supported.
     * ROS topic name: '<node_name>/footprint
     * Type and default: string, '[]'
     */    
    std::string footprint;
    
    /**
     * This parameter is overwritten if the "footprint" parameter is set.
     * Explanation: The radius of the robot in meters , this parameter should 
     * only be set for circular robots, all others should use the "footprint" 
     * parameter described above.
     * ROS topic name: '<node_name>/robot_radius
     * Type and default: double, 0.30 (0.46 in section 3.2 in ROS wiki)
     * Min, Max value: 0.0, 10.0
     */    
    double robotRadius;
    
    /*
     * Explanation: Whether or not to use the static map to initialize the 
     * costmap. If the rolling_window parameter is set to true, this parameter 
     * must be set to false. 
     * ROS topic name: '<node_name>/static_map
     * Type and default: bool, false (true in section 3.2 in ROS wiki)
     */    
    bool staticMap;
    
    /**
     * Explanation: Whether or not to use a rolling window version of the 
     * costmap. If the static_map parameter is set to true, this parameter 
     * must be set to false. 
     * ROS topic name: '<node_name>/rolling_window
     * Type and default: bool, true (false in section 3.2 in ROS wiki)
     */
    bool rollingWindow;
    
    /**
     * Explanation: The value for which a cost should be considered unknown when 
     * reading in a map from the map server. If the costmap is not tracking 
     * unknown space, costs of this value will be considered occupied. A value 
     * of zero also results in this parameter being unused. 
     * ROS topic name: '<node_name>/unknown_cost_value
     * Type and default: int, 0
     * Min, Max value: 0, 255
     */
    int unknownCostValue;
    
    /**
     * This parameter is overwritten if "static_map" is set to true, and its 
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The width of the map in meters.
     * ROS topic name: '<node_name>/width
     * Type and default: int, 6 (10 in section 3.2 in ROS wiki)
     * Min, Max value: 0, 20
     */    
    int width;
    
    /**
     * This parameter is overwritten if "static_map" is set to true, and its 
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The height of the map in meters.
     * ROS topic name: '<node_name>/height
     * Type and default: int, 6 (10 in section 3.2 in ROS wiki)
     * Min, Max value: 0, 20
     */    
    int height;
    
    /**
     * This parameter is overwritten if "static_map" is set to true, and its 
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The resolution of the map in meters/cell. 
     * ROS topic name: '<node_name>/resolution
     * Type and default: double, 0.05
     * Min, Max value: 0.0, 50.0
     */
    double resolution;
    
    /**
     * This parameter is overwritten if "static_map" is set to true, and its 
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The x origin of the map in the global frame in meters. 
     * ROS topic name: '<node_name>/origin_x
     * Type and default: double, 0.0
     * Min, Max value: 0.0, inf
     */            
    double originX;
    
    /**
     * This parameter is overwritten if "static_map" is set to true, and its 
     * original value will be restored when "static_map" is set back to false.
     * Explanation: The y origin of the map in the global frame in meters.
     * ROS topic name: '<node_name>/origin_y
     * Type and default: double, 0.0
     * Min, Max value: -inf, inf
     */        
    double originY;
    
    /**
     * Explanation: Whether or not to publish the underlying voxel grid for 
     * visualization purposes. 
     * ROS topic name: '<node_name>/publish_voxel_map
     * Type and default: bool, false
     */    
    bool publishVoxelMap;
    
    /**
     * Explanation: The threshold value at which to consider a cost lethal when 
     * reading in a map from the map server. 
     * ROS topic name: '<node_name>/lethal_cost_threshold
     * Type and default: int, 100
     * Min, Max value: 0, 255
     */    
    int lethalCostThreshold;
    
    /**
     * Explanation: The topic that the costmap subscribes to for the static map.
     * This parameter is useful when you have multiple costmap instances within 
     * a single node that you want to use different static maps. 
     * ROS topic name: '<node_name>/map_topic'.
     * Type and default: string, 'map'
     */
    std::string mapTopic;
    
    /**
     * Explanation: What map type to use. "voxel" or "costmap" are the supported
     * types, with the difference between them being a 3D-view of the world vs. 
     * a 2D-view of the world. 
     * ROS topic name: '<node_name>/map_type'.
     * Type and default: string, 'costmap' ('voxel' in section 3.2 in ROS wiki)
     */
    std::string mapType;
    
    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The z origin of the map in meters. 
     * ROS topic name: '<node_name>/origin_z'.
     * Type and default: double, 0.0
     * Min, Max value: 0.0, inf
     */
    double zOrigin;
    
    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The z resolution of the map in meters/cell.
     * ROS topic name: '<node_name>/z_resolution'.
     * Type and default: double, 0.2
     * Min, Max value: 0.0, 50.0
     */    
    double zResolution;

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The number of voxels to in each vertical column, the height 
     * of the grid is z_resolution * z_voxels.
     * ROS topic name: '<node_name>/z_voxels'.
     * Type and default: int, 10
     * Min, Max value: 0, 16
     */    
    int zVoxels;

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The number of unknown cells allowed in a column considered 
     * to be "known".
     * ROS topic name: '<node_name>/unknown_threshold'.
     * Type and default: int, 15
     * Min, Max value: 0, 16
     */    
    int unknownThreshold;

    /**
     * This parameter is only used if map_type is set to "voxel".
     * Explanation: The maximum number of marked cells allowed in a column 
     * considered to be "free". 
     * ROS topic name: '<node_name>/mark_threshold'.
     * Type and default: int, 0
     * Min, Max value: 0, 16
     */    
    int markThreshold;
        
    /**
     * This parameter is only used if map_type is set to "costmap".
     * Explanation: Specifies whether or not to track what space in the costmap 
     * is unknown, meaning that no observation about a cell has been seen from 
     * any sensor source.    
     * ROS topic name: '<node_name>/track_unknown_space'.
     * Type and default: bool, false
     */
    bool trackUnknownSpace;
    
    /**
     * Explanation: Reset all values at the target dynamic_reconfiguration 
     * server to its defaults.
     * ROS topic name: '<node_name>/restore_defaults'.
     * Type and default: bool, false
     */
    bool restoreDefaults;
    
    /**
     * Non ROS configuratin parameter!
     * Setting this parameter to true allows for manipulation of global_costmap
     * parameters. Setting this parameter to false allows for manipulation of 
     * local_costmap parameters. The default value is 'true'.
     */
    bool isGlobalCostmap;
};

}

#endif /* ROSCOSTMAPCONFIGURATION_H_ */