/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

#ifndef DOOR_H_
#define DOOR_H_

#include "../../Type.h"
#include "../person/Percept.h"
#include "../../units/UnitConverter.h"
#include "../geometry/Point2D.h"
#include "../geometry/Point3D.h"

namespace airobots {

/**
 * Door percept.
 *
 * @author lziegler
 */
class Door: public Percept {
public:
	enum CoordinateSystem {
		RobotCS, GlobalCS
	};
	static std::string csToString(CoordinateSystem cs) {
		switch (cs) {
		case RobotCS:
			return "ROBOT_CS";
		case GlobalCS:
			return "GLOBAL_CS";
		default:
			throw xmltio::ParsingException(
					"Can not generate string for CoordinateSystem Enum");
		}
	}
	static CoordinateSystem csFromString(std::string cs) {
		if (cs == "ROBOT_CS") {
			return RobotCS;
		} else if (cs == "GLOBAL_CS") {
			return GlobalCS;
		} else {
			throw xmltio::ParsingException("Can not parse coordinate system",
					cs);
		}
	}

	static std::string getBaseTag();
	static std::string getTemplate();
	static std::string getPerceptName();

	Door();
	virtual ~Door();

	Point2D getDoorLeft() const;
	void setDoorLeft(Point2D angle);

	Point2D getDoorRight() const;
	void setDoorRight(Point2D distance);

	Point3D getHandleLeft() const;
	void setHandleLeft(Point3D angleVariance);

	Point3D getHandleRight() const;
	void setHandleRight(Point3D distanceVariance);

	/**
	 * Confidence is given in percent!! (float: 0.0 ... 100.0)
	 */
	float getConfidence() const;
	void setConfidence(float confidence);

	CoordinateSystem getCoordinateSystem() const;
	void setCoordinateSystem(CoordinateSystem cs);

	bool operator==(const Door &other) const;

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, Door &type);

private:

	Point2D doorLeft;
	Point2D doorRight;
	Point3D handleLeft;
	Point3D handleRight;
	float confidence;
	CoordinateSystem cs;
};

std::ostream& operator<<(std::ostream& o, Door const& door);

}

#endif /* DOOR_H_ */
