/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke,
 * 				Dennis Leroy Wigand
 */

/*
 * Body.cpp
 *
 *  Created on: Jul 15, 2012
 *      Author: Dennis Leroy Wigand (dwigand)
 */

#include "Body.h"

#include <iostream>
#include <math.h>

using namespace std;
using namespace xmltio;
using namespace airobots;

Body::Body() {
	skel_Torso = Point3D(0, 0, 0, MILLIMETER);
	centerOfMass = Point3D(0, 0, 0, MILLIMETER);
	isSkeletonActive = false;
	kinect_id = -1;
}

Body::~Body() {
}

std::string Body::getBaseTag() {
	return "BODY";
}

std::string Body::getTemplate() {
	string sensororigin = Point3D::getTemplate();
	size_t pos = sensororigin.find(Point3D::getBaseTag());
	while (pos != sensororigin.npos) {
		sensororigin.replace(pos, Point3D::getBaseTag().length(), "COM");
		pos = sensororigin.find(Point3D::getBaseTag());
	}
	string torsoorigin = Point3D::getTemplate();
	pos = torsoorigin.find(Point3D::getBaseTag());
	while (pos != torsoorigin.npos) {
		torsoorigin.replace(pos, Point3D::getBaseTag().length(), "TORSO");
		pos = torsoorigin.find(Point3D::getBaseTag());
	}
	return string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate() + sensororigin
			+ "<SKELETON active=\"\" kinectid=\"\">" + torsoorigin
			+ "</SKELETON>" + string("</") + getBaseTag() + ">";
}

int Body::getKinectId() const {
	return kinect_id;
}

void Body::setKinectId(int id) {
	kinect_id = id;
}

std::string Body::getPerceptName() {
	return "body";
}

Point3D Body::getCoM() const {
	return this->centerOfMass;
}

void Body::setCoM(Point3D com) {
	this->centerOfMass = com;
}

Point3D Body::getTorso() const {
	return this->skel_Torso;
}

void Body::setTorso(Point3D joint) {
	this->skel_Torso = joint;
}

float Body::getAngle(AngleUnit angleUnit) const {
	float lengthCOM = getDistance(airobots::MILLIMETER);

	return UnitConverter::convert(asin((float)((float)getCoM().getX(airobots::MILLIMETER) / (float)lengthCOM)), airobots::RADIAN, angleUnit);
}

float Body::getDistance(LengthUnit lengthUnit) const {
	return UnitConverter::convert(sqrt(
			pow(getCoM().getX(airobots::MILLIMETER), 2)
					+ pow(getCoM().getZ(airobots::MILLIMETER), 2)), airobots::MILLIMETER, lengthUnit);
}

void Body::setDistanceAndAngle(float distance, LengthUnit lengthUnit, float angle, AngleUnit angleUnit) {

	float convAngle = UnitConverter::convert(angle, angleUnit, airobots::RADIAN);
	float convDist = UnitConverter::convert(distance, lengthUnit, airobots::MILLIMETER);

	centerOfMass.setX(sin(convAngle) * convDist, airobots::MILLIMETER);
	centerOfMass.setZ(cos(convAngle) * convDist, airobots::MILLIMETER);
}

bool Body::isSkeleton() const {
	return isSkeletonActive;
}

void Body::setSkeleton(bool isSkeleton) {
	isSkeletonActive = isSkeleton;
}

bool Body::operator==(const Body &other) const {
	if (other.isSkeleton() != isSkeleton()) {
		return false;
	}
	if (isSkeleton()) {
		return Type::operator==(other) && (getCoM() == other.getCoM())
				&& (getKinectId() == other.getKinectId())
				&& (getTorso() == other.getTorso());
	}
	return Type::operator==(other) && (getKinectId() == other.getKinectId())
			&& (getCoM() == other.getCoM());
}

std::ostream& airobots::operator<<(std::ostream& o, Body const& body) {
	if (body.isSkeleton()) {
		return o << "Body [com = (" << body.getCoM().getX(MILLIMETER) << ", "
				<< body.getCoM().getY(MILLIMETER) << ", "
				<< body.getCoM().getZ(MILLIMETER) << "), isSkeleton = "
				<< body.isSkeleton() << ", kinectId: " << body.getKinectId()
				<< " Torso = (" << body.getTorso().getX(MILLIMETER) << ", "
				<< body.getTorso().getY(MILLIMETER) << ", "
				<< body.getTorso().getZ(MILLIMETER) << ")" << " ]";
	}
	return o << "Body [com = (" << body.getCoM().getX(MILLIMETER) << ", "
			<< body.getCoM().getY(MILLIMETER) << ", "
			<< body.getCoM().getZ(MILLIMETER) << "), isSkeleton = "
			<< body.isSkeleton() << ", kinectId: " << body.getKinectId() << " ]";

}

void Body::sanitizeLocation(xmltio::Location &loc) const {

}

void Body::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("COM")] = getCoM();

	loc[XPath("SKELETON/@active")] = isSkeleton() ? 1 : 0;
	loc[XPath("SKELETON/@kinectid")] = getKinectId();

	if (this->isSkeleton()) {
		loc[XPath("SKELETON/TORSO")] = getTorso();
	} else {
		// delete childs to save space (good, when we send alls joints in the future!)
		loc[XPath("SKELETON")] = "";
	}

}

void Body::fromLocation(const xmltio::Location &loc_, Body &body) {

	Type::fromLocation(loc_, body);
	body.setSkeleton(
			extract<int>(loc_[XPath("SKELETON/@active")]) ? true : false);
	body.setKinectId(extract<int>(loc_[XPath("SKELETON/@kinectid")]));

	Point3D com = extract<Point3D>(loc_[XPath("COM")]);
	body.setCoM(com);

	if (body.isSkeleton()) {
		Point3D torso = extract<Point3D>(loc_[XPath("SKELETON/TORSO")]);
		body.setTorso(torso);
	}

}
