/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.asu;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.logging.Level;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import org.apache.log4j.Logger;

/**
 * Provides access to all stored frames.
 *
 * @author malinke
 */
public class FrameDB implements Iterable<Entry<String, FrameEntry>> {

	public final Logger LOGGER = Logger.getLogger(getClass());
	private Map<String, FrameEntry> frameMap = new TreeMap<String, FrameEntry>();
	private final String configFileURI;

	/**
	 * Loads the complete database.
	 *
	 * @param fileName of database file
	 */
	public FrameDB(final InputStream fileStream) {
		this.configFileURI = "constructor defined input stream";
		LOGGER.info("load " + this);
		this.loadDatabase(fileStream);
	}

	/**
	 * Loads the complete database.
	 *
	 * @param fileName of database file
	 */
	public FrameDB(final String configFileURI) {
		this.configFileURI = configFileURI;
		this.loadDatabase(configFileURI);
	}

	/**
	 * Returns the frame entry of the given name.
	 *
	 * @param frameName name of the wanted frame
	 * @return frame's entry
	 */
	public final FrameEntry getFrame(final String frameName) {
		if (!frameMap.containsKey(frameName)) {
			// generate frames without slot definitions in config file 
			frameMap.put(frameName, new FrameEntry(frameName));
		}
		return frameMap.get(frameName);
	}

	/**
	 * Returns the names of all frames.
	 *
	 * @return names of all frames
	 */
	public final String[] getFrameNames() {
		String[] frameNames = new String[frameMap.size()];
		return frameMap.keySet().toArray(frameNames);
	}

	/**
	 * {@inheritDoc}
	 */
	public final Iterator<Entry<String, FrameEntry>> iterator() {
		return frameMap.entrySet().iterator();
	}

	private void loadDatabase(final String filename) {
		Builder b = new Builder();
		try {
			Document dbDoc = b.build(filename);
			Elements entries = dbDoc.getRootElement().getChildElements("frame");
			createEntries(entries);
			connectFrames(entries);
		} catch (ParsingException ex) {
			throw new IllegalArgumentException("Could not parse db file: " + ex);
		} catch (IOException ex) {
			throw new IllegalArgumentException("Could not read db file: " + ex);
		}
	}

	private void loadDatabase(final InputStream fileStream) {
		Builder b = new Builder();
		try {
			Document dbDoc = b.build(fileStream);
			Elements entries = dbDoc.getRootElement().getChildElements("frame");
			createEntries(entries);
			connectFrames(entries);
		} catch (ParsingException ex) {
			throw new IllegalArgumentException("Could not parse db file: " + ex);
		} catch (IOException ex) {
			throw new IllegalArgumentException("Could not read db file: " + ex);
		}
	}

	private void createEntries(final Elements entries) {
		for (int i = 0; i < entries.size(); i++) {
			Element entry = entries.get(i);
			getFrame(entry.getAttributeValue("name"));
		}
	}

	private void connectFrames(final Elements entries) {
		for (int i = 0; i < entries.size(); i++) {
			Element entry = entries.get(i);
			String name = entry.getAttributeValue("name");
			assert frameMap.get(name) != null;
			FrameEntry frame = frameMap.get(name);
			connectTopFrame(frame, entry);
			connectSlots(frame, entry);
		}
	}

	private void connectTopFrame(final FrameEntry frame, final Element entry) {
		if (entry.getAttribute("topFrame") == null) {
			return;
		}
		String topFrameName = entry.getAttributeValue("topFrame");
		frame.setTopFrame(getFrame(topFrameName));
	}

	private void connectSlots(final FrameEntry frame, final Element entry) {
		Elements slots = entry.getChildElements("slot");
		for (int i = 0; i < slots.size(); i++) {
			try {
				frame.addSlot(new Slot(slots.get(i), this));
			} catch (ParsingException ex) {
				LOGGER.warn("Could not parse slot!", ex);
			}
		}
	}

	@Override
	public String toString() {
		return getClass().getSimpleName() + "[config:" + configFileURI + "]";
	}
}
