/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.map;

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.navigation.PositionData;
import java.util.UUID;
import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.ParsingException;

/**
 * Representation of a viewpoint. Each viewpoint has coordinates (x, y, yaw), a
 * prescribed category and a unique label.
 * 
 * <code>
 * <VIEWPOINT label="Bottle" category="OBJECT">
 *      <TIMESTAMP>
 *          <INSERTED value="1334740313587"/>
 *          <UPDATED value="1334740313587"/>
 *      </TIMESTAMP>
 * 
 *      <GENERATOR>unknown</GENERATOR>
 * 
 *      <ROBOTPOSITION>
 *          <TIMESTAMP>
 *              <INSERTED value="1334589075983" />
 *              <UPDATED value="1334589075983" />
 *          </TIMESTAMP>
 *          <GENERATOR>unknown</GENERATOR>
 *          <POSITION x="0.0" y="1.0" theta="2.0" ref="world" kind="absolute" />
 *      </ROBOTPOSITION>
 * </VIEWPOINT>
 * </code>
 *
 * @author lkettenb
 */
public class Viewpoint extends Type {

    /**
     * Three categories that a viewpoint can belong to.
     */
    public enum ViewpointCategory {

        PERSON, OBJECT, VIEW
    }
    /**
     * Coordinates of this viewpoint (x, y, yaw).
     */
    private PositionData coordinates = new PositionData();
    /**
     * The (optional) category of this viewpoint. Default: {@link ViewpointCategory#VIEW}.
     *
     * @see ViewpointCategory
     */
    private ViewpointCategory category = ViewpointCategory.VIEW;
    /**
     * The label of this viewpoint. This label should be unique. Please note
     * that this is not verified until now.
     */
    private String label = UUID.randomUUID().toString();
    
    /**
     * Default constructor as expected by {@link Type}. You should not use it!
     */
    public Viewpoint() {
        
    }
    
    /**
     * Constructor.
     *
     * @param coordinates Coordinates of the viewpoint (x, y, yaw).
     */
    public Viewpoint(PositionData coordinates) {
        this.coordinates = coordinates;
    }

    /**
     * Constructor.
     *
     * @param coordinates Coordinates of the viewpoint (x, y, yaw).
     * @param label The unique label of this viewpoint.
     *
     * @see ViewpointCategory
     */
    public Viewpoint(PositionData coordinates,
            String label) {
        this(coordinates);
        this.label = label;
    }

    /**
     * Constructor.
     *
     * @param coordinates Coordinates of the viewpoint (x, y, yaw).
     * @param category Category of this viewpoint.
     * @param label The unique label of this viewpoint.
     *
     * @see ViewpointCategory
     */
    public Viewpoint(PositionData coordinates,
            ViewpointCategory category,
            String label) {
        this(coordinates, label);
        this.category = category;
    }

    public ViewpointCategory getCategory() {
        return category;
    }

    public void setCategory(ViewpointCategory category) {
        this.category = category;
    }

    /**
     * Vector with x, y and yaw.
     *
     * @return Vector with x, y and yaw.
     */
    public PositionData getCoordinates() {
        return coordinates;
    }

    /**
     * Vector with x, y and yaw.
     *
     * @param coordinates Vector with x, y and yaw.
     */
    public void setCoordinates(PositionData coordinates) {
        this.coordinates = coordinates;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    /**
     * Getter for the XML base tag used for (de-)serialization.
     *
     * @return XML base tag.
     */
    public static String getBaseTag() {
        return "VIEWPOINT";
    }

    /**
     * Constructs a {@link Viewpoint} object from a given XOM {@link Element}.
     *
     * @param element The XOM {@link Element} to construct an object from.
     *
     * @return The {@link Viewpoint} object containing all the information given
     * by the {@link Element} object.
     */
    public static Viewpoint fromElement(Element element) {
        String tmpLabel = "";
        ViewpointCategory tmpCategory = ViewpointCategory.VIEW;
        PositionData tmpCoordinates = new PositionData();

        try {
            if (!element.getLocalName().equals(getBaseTag())) {
                throw new IllegalArgumentException("Parsing Error: Base tag "
                        + "for viewpoint object must be '" + getBaseTag() + "'");
            }
            tmpLabel = ElementParser.getAttributeValue(element, "label");
            
            Element elementCoordinates = element.getFirstChildElement(
                    PositionData.getBaseTag());
            PositionData.fromElement(elementCoordinates, tmpCoordinates);
            
            String stringCategory = 
                    ElementParser.getAttributeValue(element, "category");
            if (stringCategory.equals(ViewpointCategory.VIEW.toString())) {
                tmpCategory = ViewpointCategory.VIEW;
            } else if (stringCategory.equals(ViewpointCategory.OBJECT.toString())) {
                tmpCategory = ViewpointCategory.OBJECT;
            } else if (stringCategory.equals(ViewpointCategory.PERSON.toString())) {
                tmpCategory = ViewpointCategory.PERSON;
            }
        } catch (ParsingException ex) {
            throw new IllegalArgumentException("Missing element or attribute "
                    + "in document (PositionData?).", ex);
        } catch (NullPointerException ex) {
            // this happens when an element or attribute that is required is
            // not present
            throw new IllegalArgumentException("Missing element or attribute "
                    + "in document.", ex);
        }
        return new Viewpoint(tmpCoordinates, tmpCategory, tmpLabel);
    }
    
    /**
     * Constructs a {@link Viewpoint} object from a given XOM {@link Element}.
     * 
     * @param element The XOM {@link Element} to construct an object from.
     * @param viewpoint The {@link Viewpoint} object to be filled with all the 
     * information given by the {@link Element} object.
     */
    public static void fromElement(Element element, Viewpoint viewpoint) {
        Viewpoint parsedViewpoint = fromElement(element);
        viewpoint.setLabel(parsedViewpoint.getLabel());
        viewpoint.setCategory(parsedViewpoint.getCategory());
        viewpoint.setCoordinates(parsedViewpoint.getCoordinates());
    }

    /**
     * Serializes this {@link Viewpoint} object into a given XOM
     * {@link Element}.
     *
     * @param parent The {@link Element} to serialize the object into. The given
     *            {@link Element} object should have the base tag defined by this class.
     * (see {@link #getBaseTag()})
     * @see #getBaseTag()
     */
    @Override
    public void fillInto(Element parent) {
        sanitizeElement(parent);
        
        parent.addAttribute(new Attribute("label", label));
        parent.addAttribute(new Attribute("category", category.toString()));
        
        // timestamp
		Element timestampElement = new Element(Timestamp.getBaseTag());
		getTimestamp().fillInto(timestampElement);
		parent.appendChild(timestampElement);

		// generator
		Element generatorElement = new Element("GENERATOR");
		generatorElement.appendChild(getGenerator());
		parent.appendChild(generatorElement);
        
        Element item = new Element(PositionData.getBaseTag());
        coordinates.fillInto(item);
        parent.appendChild(item);
    }
    
    @Override
    public boolean equals(Object obj) {
        try {
            if (!(obj instanceof Viewpoint)) {
                return false;
            }

            Viewpoint other = (Viewpoint) obj;

            if (!other.getLabel().equals(getLabel())) {
                return false;
            }
            if (!other.getCategory().equals(getCategory())) {
                return false;
            }
            if (!other.getCoordinates().equals(getCoordinates())) {
                return false;
            }
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 29 * hash + (this.coordinates != null ? this.coordinates.hashCode() : 0);
        hash = 29 * hash + (this.category != null ? this.category.hashCode() : 0);
        hash = 29 * hash + (this.label != null ? this.label.hashCode() : 0);
        return hash;
    }
}
