/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.object;

import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.geometry.Point2D;
import de.unibi.airobots.btl.data.geometry.Point3D;

/**
 * Door percept.
 * 
 * @author: lziegler <lziegler@techfak.uni-bielefeld.de>
 */
public class Door extends Type {

	enum CoordinateSystem {
		ROBOT_CS, GLOBAL_CS
	}

	private Point2D left;
	private Point2D right;
	private Point3D handleLeft;
	private Point3D handleRight;
	private float confidence;
	private CoordinateSystem cs = CoordinateSystem.ROBOT_CS;

	public Door() {

	}

	public CoordinateSystem getCoordinateSystem() {
		return cs;
	}

	public void setCoordinateSystem(CoordinateSystem cs) {
		this.cs = cs;
	}

	public Point2D getDoorLeft() {
		return left;
	}

	public void setDoorLeft(Point2D left) {
		this.left = left;
	}

	public Point2D getDoorRight() {
		return right;
	}

	public void setDoorRight(Point2D right) {
		this.right = right;
	}

	public Point3D getHandleLeft() {
		return handleLeft;
	}

	public void setHandleLeft(Point3D handleLeft) {
		this.handleLeft = handleLeft;
	}

	public Point3D getHandleRight() {
		return handleRight;
	}

	public void setHandleRight(Point3D handleRight) {
		this.handleRight = handleRight;
	}
	
	public float getConfidence() {
		return confidence;
	}

	public void setConfidence(float confidence) {
		this.confidence = confidence;
	}

	public static String getBaseTag() {
		return "DOOR";
	}

	/**
	 * @throws ParsingException
	 * @{inheritDoc
	 */
	public static void fromElement(Element element, Door door)
			throws ParsingException {

		Type.fromElement(element, door);

		Point2D p0 = new Point2D();
		Point2D.fromElement(element.getFirstChildElement("LEFT")
				.getFirstChildElement(Point2D.getBaseTag()), p0);
		Point2D p1 = new Point2D();
		Point2D.fromElement(element.getFirstChildElement("RIGHT")
				.getFirstChildElement(Point2D.getBaseTag()), p1);
		Point3D p2 = new Point3D();
		Point3D.fromElement(element.getFirstChildElement("HANDLELEFT")
				.getFirstChildElement(Point3D.getBaseTag()), p2);
		Point3D p3 = new Point3D();
		Point3D.fromElement(element.getFirstChildElement("HANDLERIGHT")
				.getFirstChildElement(Point3D.getBaseTag()), p3);

		door.setDoorLeft(p0);
		door.setDoorRight(p1);
		door.setHandleLeft(p2);
		door.setHandleRight(p3);

		CoordinateSystem cs = CoordinateSystem.valueOf(element
				.getFirstChildElement("COORDINATE_SYSTEM").getValue());
		door.setCoordinateSystem(cs);
		door.setConfidence(Float.parseFloat(element
				.getFirstChildElement("CONFIDENCE").getValue()));
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Elements lefts = parent.getChildElements("LEFT");
		for (int i = 0; i < lefts.size(); i++) {
			parent.removeChild(lefts.get(i));
		}
		Elements rights = parent.getChildElements("RIGHT");
		for (int i = 0; i < rights.size(); i++) {
			parent.removeChild(rights.get(i));
		}
		Elements hlefts = parent.getChildElements("HANDLELEFT");
		for (int i = 0; i < hlefts.size(); i++) {
			parent.removeChild(hlefts.get(i));
		}
		Elements hrights = parent.getChildElements("HANDLERIGHT");
		for (int i = 0; i < hrights.size(); i++) {
			parent.removeChild(hrights.get(i));
		}

		Element leftEl = new Element("LEFT");
		Element leftElP = new Element(Point2D.getBaseTag());
		left.fillInto(leftElP);
		leftEl.appendChild(leftElP);
		parent.appendChild(leftEl);
		Element rightEl = new Element("RIGHT");
		Element rightElP = new Element(Point2D.getBaseTag());
		right.fillInto(rightElP);
		rightEl.appendChild(rightElP);
		parent.appendChild(rightEl);
		Element hleftEl = new Element("HANDLELEFT");
		Element hleftElP = new Element(Point3D.getBaseTag());
		handleLeft.fillInto(hleftElP);
		hleftEl.appendChild(hleftElP);
		parent.appendChild(hleftEl);
		Element hrightEl = new Element("HANDLERIGHT");
		Element hrightElP = new Element(Point3D.getBaseTag());
		handleRight.fillInto(hrightElP);
		hrightEl.appendChild(hrightElP);
		parent.appendChild(hrightEl);

		if (parent.getChildElements("COORDINATE_SYSTEM").size() == 0) {
			Element e = new Element("COORDINATE_SYSTEM");
			parent.appendChild(e);
		}

		parent.getFirstChildElement("COORDINATE_SYSTEM").appendChild(cs.name());
		
		if (parent.getChildElements("CONFIDENCE").size() == 0) {
			Element e = new Element("CONFIDENCE");
			parent.appendChild(e);
		}

		parent.getFirstChildElement("CONFIDENCE").appendChild(Float.toString(confidence));
	}
}
