/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Robin Schiewer, Sebastian 
 * Meyer zu Borgsen, Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Tristan 
 * Walter, Viktor Losing, Viktor Richter
 */
package de.unibi.airobots.btl.data.vision3d;

import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.data.geometry.Point3D;
import de.unibi.airobots.btl.data.geometry.PrecisePolygon;
import de.unibi.airobots.btl.data.geometry.Rotation3D;
import de.unibi.airobots.btl.data.object.ObjectLocationData;
import de.unibi.airobots.btl.data.object.ObjectLocationData.Hypothesis;
import de.unibi.airobots.btl.data.vision2d.RegionData;
import de.unibi.airobots.btl.data.vision2d.RegionData.Scope;

/**
 * This type represents a plane in 3D space. It may be an infinite plane (if no
 * borders are specified) or a bordered plane if a polygon describing the
 * borders is specified.
 * 
 * @author lziegler
 */
public class PlaneData extends RegionData {

	public static final String ORIGIN_TAG_NAME = "ORIGIN";
	public static final String ROTATION_TAG_NAME = "ROTATION3D";

	private Point3D origin;
	private Rotation3D rotation;

	/**
	 * Creates a new instance.
	 */
	public PlaneData() {
		super();
		origin = new Point3D();
		rotation = new Rotation3D();
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param polygon
	 *            The borders of the plane.
	 */
	public PlaneData(PrecisePolygon polygon) {
		super(polygon);
		origin = new Point3D();
		rotation = new Rotation3D();
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param origin
	 *            The origin of the plane
	 * @param rotation
	 *            The rotation of the plane's axes to the coordinate system's
	 *            axes. The z-axis is the normal of the plane
	 * @param polygon
	 *            The borders of the plane defined in the x-y axes of the plane
	 */
	public PlaneData(Point3D origin, Rotation3D rotation, PrecisePolygon polygon) {
		super(polygon);
		this.origin = origin;
		this.rotation = rotation;
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param polygon
	 *            The borders of the plane defined in the x-y axes of the plane
	 * @param scope
	 *            The scope of the coordinates.
	 */
	public PlaneData(PrecisePolygon polygon, Scope scope) {
		super(polygon, scope);
		origin = new Point3D();
		rotation = new Rotation3D();
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param origin
	 *            The origin of the plane
	 * @param rotation
	 *            The rotation of the plane's axes to the coordinate system's
	 *            axes. The z-axis is the normal of the plane
	 * @param polygon
	 *            The borders of the plane defined in the x-y axes of the plane
	 * @param scope
	 *            The scope of the coordinates.
	 */
	public PlaneData(Point3D origin, Rotation3D rotation,
			PrecisePolygon polygon, Scope scope) {
		super(polygon, scope);
		this.origin = origin;
		this.rotation = rotation;
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param origin
	 *            The origin of the plane
	 * @param rotation
	 *            The rotation of the plane's axes to the coordinate system's
	 *            axes. The z-axis is the normal of the plane
	 * @param scope
	 *            The scope of the coordinates.
	 */
	public PlaneData(Point3D origin, Rotation3D rotation, Scope scope) {
		super(scope);
		this.origin = origin;
		this.rotation = rotation;
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param origin
	 *            The origin of the plane
	 * @param rotation
	 *            The rotation of the plane's axes to the coordinate system's
	 *            axes. The z-axis is the normal of the plane
	 */
	public PlaneData(Point3D origin, Rotation3D rotation) {
		super();
		this.origin = origin;
		this.rotation = rotation;
	}

	/**
	 * Getter for the plane's origin.
	 * 
	 * @return the plane's origin.
	 */
	public Point3D getOrigin() {
		return origin;
	}

	/**
	 * Setter for the plane's origin.
	 * 
	 * @param origin
	 *            the plane's origin.
	 */
	public void setOrigin(Point3D origin) {
		this.origin = origin;
	}

	/**
	 * Getter for the rotation of the plane's axes to the coordinate system.
	 * 
	 * @return the plane's rotation.
	 */
	public Rotation3D getRotation() {
		return rotation;
	}

	/**
	 * Setter for the rotation of the plane's axes to the coordinate system.
	 * 
	 * @param rotation
	 *            the plane's rotation.
	 */
	public void setRotation(Rotation3D rotation) {
		this.rotation = rotation;
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "PLANE";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// cleanup
		Elements elements = parent.getChildElements(ORIGIN_TAG_NAME);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}
		elements = parent.getChildElements(ROTATION_TAG_NAME);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}

		// fill
		Element originElement = new Element(ORIGIN_TAG_NAME);
		origin.fillInto(originElement);
		parent.appendChild(originElement);

		Element normalElement = new Element(ROTATION_TAG_NAME);
		rotation.fillInto(normalElement);
		parent.appendChild(normalElement);
	}

	/**
	 * Fills an {@link ObjectLocationData.Hypothesis} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param hypothesisElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectLocationData.Hypothesis} object to fill with
	 *            all the information given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element dataElement, PlaneData type)
			throws ParsingException {

		try {
			RegionData.fromElement(dataElement, type);

			Element originElement = dataElement
					.getFirstChildElement(ORIGIN_TAG_NAME);
			Point3D origin = new Point3D();
			Point3D.fromElement(originElement, origin);
			type.setOrigin(origin);

			Element normalElement = dataElement
					.getFirstChildElement(ROTATION_TAG_NAME);
			if (normalElement == null) {
				throw new ParsingException("Missing tag " + ROTATION_TAG_NAME);
			}
			Rotation3D rotation = new Rotation3D();
			Rotation3D.fromElement(normalElement, rotation);
			type.setRotation(rotation);
		} catch (IllegalArgumentException e) {
			throw new ParsingException(e.getMessage(), e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[" + getBaseTag() + " timestamp: " + getTimestamp()
				+ " scope: " + scope + " Origin: " + origin + " Rotation: "
				+ rotation + " Polygon: " + polygon + "]";

	}
}
